/*******************************************************************************
 * Copyright (c) 2001, 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.soda.sat.core.util;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.eclipse.soda.sat.core.internal.util.Assert;
import org.osgi.framework.Constants;
import org.osgi.framework.ServiceReference;

/**
 * The <code>ServiceReferenceUtility</code> class is a utility that simplifies
 * working with <code>ServiceReference</code> objects.
 * <p>
 * <i>Note:</i> This is a singleton class, accessible only via the static method
 * <code>getInstance()</code>.  For example:
 * <pre>
 * ServiceReferenceUtility utility = ServiceReferenceUtility.getInstance();
 * String[] names = utility.getServiceNames(reference);
 * </pre>
 *
 * @see org.osgi.framework.ServiceReference
 */
public final class ServiceReferenceUtility extends Object {
	//
	// Static Fields
	//

	// Singleton
	private static final ServiceReferenceUtility INSTANCE = new ServiceReferenceUtility();

	//
	// Static Methods
	//

	/**
	 * Public getter for the <code>ServiceReferenceUtility</code> singleton
	 * instance.
	 *
	 * @return The <code>ServiceReferenceUtility</code> singleton instance.
	 */
	public static ServiceReferenceUtility getInstance() {
		return ServiceReferenceUtility.INSTANCE;
	}

	/**
	 * Constructor.  Instances of this class cannot be created.
	 */
	private ServiceReferenceUtility() {
		super();
	}

	//
	// Instance Methods
	//

	/**
	 * Find the index of the first occurrence of the highest rank.
	 *
	 * @param ranks  An array of ranks.
	 *
	 * @return The index of the first occurrence of the highest rank, or -1 if
	 * the array of ranks is empty.
	 */
	private int getIndexOfHighestServiceRank(int[] ranks) {
		int length = ranks.length;
		if (length == 0) return -1;  // Early return.

		int index = 0;
		int max = Integer.MIN_VALUE;

		for (int i = 0 ; i < length; i++) {
			int rank = ranks [ i ];

			if (rank > max) {
				index = i;
				max = rank;
			}
		}

		return index;
	}

	/**
	 * Find the index of the lowest <code>Constants.SERVICE_ID</code> from an
	 * array of service IDs.  The only ids that are looked at are those that are
	 * tied with the specified rank.
	 *
	 * @param ranks  An array of ranks that contains tied ranks.
	 * @param start  The index into the array of ranks at which to start
	 *               searching for tied ranks.
	 * @param ids    An array of service IDs from which to pick the lowest for
	 *               a tied rank.
	 * @return The index of the lowest service ID for the tied rank.
	 */
	private int getIndexOfLowestServiceId(int[] ranks, long[] ids, int start) {
		int index = start;
		int length = ids.length;
		int rank = ranks [ start ];
		long min = ids [ start ];

		for (int i = start + 1; i < length; i++) {
			if (ranks [ i ] == rank) {
				long id = ids [ i ];

				if (id < min) {
					index = i;
					min = id;
				}
			}
		}

		return index;
	}

	/**
	 * Gets the <code>Constants.SERVICE_ID</code> property from the specified
	 * <code>ServiceReference</code> objects.
	 *
	 * @param reference  A <code>ServiceReference</code>.
	 * @return The <code>Constants.SERVICE_ID</code> property value.
	 */
	public long getServiceId(ServiceReference reference) {
		Assert.checkArgumentIsNotNull(reference, "reference");  //$NON-NLS-1$
		Number wrapper = (Number) reference.getProperty(Constants.SERVICE_ID);
		long id = wrapper.intValue();
		return id;
	}

	/**
	 * Gets the <code>Constants.SERVICE_ID</code> properties from an array of
	 * <code>ServiceReference</code> objects.
	 *
	 * @param references  An array of <code>ServiceReference</code> objects.
	 * @return An array of <code>Constants.SERVICE_ID</code> property values.
	 */
	public long[] getServiceIds(ServiceReference[] references) {
		Assert.checkArgumentIsNotNull(references, "references");  //$NON-NLS-1$
		int length = references.length;
		long[] ids = new long [ length ];

		for (int i = 0; i < length; i++) {
			ServiceReference reference = references [ i ];
			long id = getServiceId(reference);
			ids [ i ] = id;
		}

		return ids;
	}

	/**
	 * Answers an array of service names available from the specified
	 * <code>ServiceReference</code>.
	 *
	 * @param reference  The <code>ServiceReference<code> to be queried.
	 * @return A list of fully qualifed type names.
	 */
	public List/*<String>*/ getServiceNames(ServiceReference reference) {
		Assert.checkArgumentIsNotNull(reference, "reference");  //$NON-NLS-1$
		String[] names = (String[]) reference.getProperty(Constants.OBJECTCLASS);
		List/*<String>*/ list = new ArrayList/*<String>*/(names.length);

		for (int i = 0; i < names.length; i++) {
			String name = names [ i ];
			list.add(name);
		}

		return list;
	}

	/**
	 * Gets the service names from an array of <code>ServiceReference</code>
	 * objects.
	 *
	 * @param references  An array of <code>ServiceReference</code> objects.
	 * @return An list of service names.
	 */
	public List/*<String>*/ getServiceNames(ServiceReference[] references) {
		Assert.checkArgumentIsNotNull(references, "references");  //$NON-NLS-1$
		MiscUtility utility = MiscUtility.getInstance();
		int size = utility.estimateHashedCollectionSize(references.length * 4);
		Set/*<String>*/ set = new HashSet/*<String>*/(size);

		for (int i = 0; i < references.length; i++) {
			ServiceReference reference = references [ i ];
			List/*<String>*/ names = getServiceNames(reference);
			set.addAll(names);
		}

		int count = set.size();
		ArrayList/*<String>*/ list = new ArrayList/*<String>*/(count);
		list.addAll(set);
		return list;
	}

	/**
	 * Gets the <code>Constants.SERVICE_RANKING</code> properties from an array
	 * of <code>ServiceReference</code> objects.
	 *
	 * @param references  An array of <code>ServiceReference</code> objects.
	 * @return An array of <code>Constants.SERVICE_RANKING</code> property
	 * values.
	 */
	private int[] getServiceRanks(ServiceReference[] references) {
		int length = references.length;
		int[] ranks = new int [ length ];

		for (int i = 0; i < length; i++) {
			ServiceReference reference = references [ i ];
			Object value = reference.getProperty(Constants.SERVICE_RANKING);
			int ranking;

			if (value instanceof Integer) { // $codepro.audit.disable disallowInstanceof
				Number wrapper = (Number) value;
				ranking = wrapper.intValue();
			} else {
				ranking = 0;
			}

			ranks [ i ] = ranking;
		}

		return ranks;
	}

	/**
	 * Answers true if the <code>ServiceReference</code> is an instance of the
	 * service name.
	 *
	 * @param reference  The <code>ServiceReference</code> to be queried.
	 * @param name       A fully qualified type name of a service.
	 * @return If the <code>ServiceReference</code> represents the service
	 * name return <code>true</code>, otherwise <code>false</code>.
	 */
	public boolean isServiceInstanceOf(ServiceReference reference, String name) {
		Assert.checkArgumentIsNotNull(reference, "reference");  //$NON-NLS-1$
		Assert.checkArgumentIsNotNull(name, "name");  //$NON-NLS-1$
		boolean match = false;
		List/*<String>*/ serviceNames = getServiceNames(reference);
		Iterator/*<String>*/ iterator = serviceNames.iterator();

		while (match == false && iterator.hasNext() == true) {
			String serviceName = (String) iterator.next();
			match = name.equals(serviceName);
		}

		return match;
	}

	/**
	 * Tests to see if a rank is tied with another rank.
	 *
	 * @param ranks  An array of ranks that might contain a tie.
	 * @param start  The index into the array of ranks at which to start
	 *               searching for a tie.
	 * @return If a tied rank exists return <code>true</code>, otherwise
	 * <code>false</code>.
	 */
	private boolean isServiceRankTied(int[] ranks, int start) {
		boolean tied = false;
		int rank = ranks [ start ];
		int index = start + 1;
		int length = ranks.length;

		while (tied == false && index < length) {
			tied = ranks [ index ] == rank;
			index++;
		}

		return tied;
	}

	/**
	 * Select an appropriate <code>ServiceReference</code> from an array.  The
	 * criteria for selection is as follows:
	 * <ul>
	 *   <li>
	 *       The <code>ServiceReference</code> with the highest ranking as
	 *       specified in its <code>Constants.SERVICE_RANKING</code> property is
	 *       selected.
	 *   </li>
	 *   <li>
	 *       If there is a tie in ranking, the <code>ServiceReference</code>
	 *       with the lowest service ID, as specified in its
	 *       <code>Constants.SERVICE_ID</code> property, is selected.
	 *   </li>
	 * </ul>
	 *
	 * @param references  An array of <code>ServiceReference</code> objects from
	 *                    which to select.
	 * @return The matching <code>ServiceReference</code> or <code>null</code>.
	 */
	public ServiceReference select(ServiceReference[] references) {
		Assert.checkArgumentIsNotNull(references, "references");  //$NON-NLS-1$
		int[] ranks = getServiceRanks(references);
		int index = getIndexOfHighestServiceRank(ranks);
		if (index == -1) return null;  // Early return.

		boolean tied = isServiceRankTied(ranks, index);

		if (tied == true) {
			long[] ids = getServiceIds(references);
			index = getIndexOfLowestServiceId(ranks, ids, index);
		}

		ServiceReference result = references [ index ];
		return result;
	}
}